/*!@file
** @brief   The entry point of the program.
** @remarks This file is the single compilation unit of the program.
** @author  UETA, Shin-ichi <ueta@usdesign.jp>
**
** @copyright (C) 2014-2019 Ueta System Design Studio\n
** This file is licensed under the terms of the MIT License.\n
** See <https://opensource.org/licenses/MIT>.
*/
#include "stdafx.h"
#include "resource.h"
#include "about.h"
#include "clipboard.h"

/*!
** @brief   The global instance of CComModule.
** @remarks -
*/
CComModule _Module;

/*!
** @brief   Implements the entry point for the application.
** @param   [in] hInstance a handle to the current instance of the application.
** @param   [in] hPrevInstance a legacy from 16-bit Windows, now always zero.
** @param   [in] lpCmdLine the command-line for the application.
** @param   [in] nCmdShow specifies how the window is to be shown.
** @returns A standard HRESULT value as the exit value.
** @remarks -
*/
extern "C" int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hInstance);
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);
	UNREFERENCED_PARAMETER(nCmdShow);

	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);

	HRESULT hResult = S_OK;

	hResult = ::CoInitialize(NULL);
	ATLASSERT(SUCCEEDED(hResult));
	::DefWindowProc(NULL, WM_NULL, 0, 0L);
	ATLASSERT(!CAtlBaseModule::m_bInitFailed);
	hResult = _Module.Init(NULL, hInstance);
	ATLASSERT(SUCCEEDED(hResult));

	#if 1 // Setup
	{
		;
	}
	#endif

	#if 1 // Test 1: CClipboard::SetText
	{
		const wchar_t* pszExpected = L"Hello, world!";

		ATLVERIFY(CClipboard::SetText(pszExpected));

		ATLVERIFY(::OpenClipboard(NULL));

		int nAvailableFormats = 0;

		for (UINT uFormat = ::EnumClipboardFormats(0); uFormat; uFormat = ::EnumClipboardFormats(uFormat))
		{
			switch (uFormat)
			{
			case CF_LOCALE:
			case CF_OEMTEXT:
			case CF_TEXT:
			case CF_UNICODETEXT:

				ATLASSERT(::IsClipboardFormatAvailable(uFormat));
				break;

			default:

				ATLASSERT(FALSE);
				break;
			}
			++nAvailableFormats;
		}
		ATLASSERT(nAvailableFormats == 4);
		ATLASSERT(::CountClipboardFormats() == nAvailableFormats);

		HGLOBAL hGlobal = ::GetClipboardData(CF_UNICODETEXT);
		ATLASSERT(hGlobal);
		wchar_t* pszGlobal = static_cast<wchar_t*>(::GlobalLock(hGlobal));
		ATLASSERT(pszGlobal);
		ATLTRACE(TEXT("\"%ls\"\n"), pszGlobal);
		ATLASSERT(wcscmp(pszGlobal, pszExpected) == 0);
		if (::GlobalUnlock(hGlobal) == 0)
			ATLASSERT(::GetLastError() == NO_ERROR);

		ATLVERIFY(::CloseClipboard());
	}
	#endif

	#if 1 // Test 2:
	{
		;
	}
	#endif

	#if 1 // Teardown
	{
		;
	}
	#endif

	_Module.Term();
	::CoUninitialize();
	return int(hResult);
}
